﻿using UnityEngine;

namespace Overdash.Common
{
    public class MonoBehaviourSingleton<T> : MonoBehaviour where T : MonoBehaviour
    {
        public static T instance { get; private set; }
        
        protected virtual void Awake()
        {
            if (instance == null)
                instance = this as T;
            else
            {
                Debug.LogWarning($"Multiple instances of {typeof(T)} found. Destroying this instance.");    
                Destroy(gameObject);
            }
                
        }
    }
}