using System.Collections.Generic;
using Overdash.Combat;
using Overdash.Global;
using UnityEngine;

namespace Overdash.Levels
{
    [RequireComponent(typeof(Grid))]
    public class Level : MonoBehaviour
    {
        [field: SerializeField] public string levelName { get; private set; }
        
        [field: SerializeField] public bool customStartingAmmoEnabled { get; private set; }
        [field: SerializeField] public List<AmmoType> customStartingAmmo { get; private set; }
        
        [HideInInspector] public List<SpawnPoint> spawnPoints;
    
        private void Awake()
        {
            spawnPoints = new List<SpawnPoint>(GetComponentsInChildren<SpawnPoint>());
        
            // Spawn points count check
            switch (spawnPoints.Count)
            {
                case < 4:
                    Debug.LogWarning($"[Level] Less than 4 spawn points in level {levelName}!");
                    break;
                case < 8:
                    Debug.Log("[Level] Less than 8 spawn points found in level " + levelName);
                    break;
                case > 8:
                    Debug.LogWarning($"[Level] More than 8 spawn points in level {levelName}!");
                    break;
            }

            // Reset spawn points on round end
            RoundLoop.OnRoundEnd += () =>
            {
                foreach (var spawnPoint in spawnPoints)
                    spawnPoint.isOccupied = false;
            };
        }
    }
}
