﻿using System;
using System.Collections;
using System.Collections.Generic;
using Overdash.Player;
using UnityEngine;
using TMPro;

namespace Overdash.UI
{
    [RequireComponent(typeof(CanvasGroup))]
    public class EndOfRoundScreen : MonoBehaviour
    {
        [SerializeField] private GameObject playerScoreUIPrefab;
        [SerializeField] private Transform playerListNode;
        [SerializeField] private TextMeshProUGUI roundEndText;
        [SerializeField] private int maxPoints = 5;
        
        private readonly Dictionary<int, PlayerScoreUI> playerScoreUIs = new ();
        private CanvasGroup canvasGroup;

        private void Start()
        {
            canvasGroup = GetComponent<CanvasGroup>();
            
            PlayerManager.OnPlayerJoined += AddPlayer;
            foreach (var player in PlayerManager.players)
                AddPlayer(player);
        }

        private void AddPlayer(Player.Player player)
        {
            var playerScoreUI = Instantiate(playerScoreUIPrefab, playerListNode).GetComponent<PlayerScoreUI>();
            playerScoreUI.SetPlayer(player);
            playerScoreUIs.Add(player.number, playerScoreUI);
        }

        private bool Result(IEnumerable<PlayerDeathInfo> killEvents)
        {
            int winners = 0;
            roundEndText.text = $"ROUND END!";
            
            foreach (var killEvent in killEvents)
            {
                if ((killEvent.KilledPlayer != killEvent.KilledByPlayer) && 
                    (playerScoreUIs[killEvent.KilledByPlayer.number].GetScore() == (maxPoints - 1))){
                        winners++;
                        roundEndText.text = $"P{killEvent.KilledByPlayer.number} WINS!";
                }    
            }

            if (winners > 1){
                roundEndText.text = $"DRAW!";
            }

            return winners > 0;
        }

        private void ResetScore()
        {
            foreach (KeyValuePair<int, PlayerScoreUI> playerScore in playerScoreUIs){
                    playerScore.Value.SetScore(0);
                }
        }

        public IEnumerator Run(IEnumerable<PlayerDeathInfo> killEvents)
        {
            bool playerWin = Result(killEvents);

            // Fade in
            while (canvasGroup.alpha < 1)
            {
                canvasGroup.alpha += Time.unscaledDeltaTime / 0.25f;
                yield return null;
            }   
            
            yield return new WaitForSecondsRealtime(0.5f);
            foreach (var killEvent in killEvents)
            {
                // Suicide
                if (killEvent.KilledPlayer == killEvent.KilledByPlayer)
                    playerScoreUIs[killEvent.KilledPlayer.number].RemovePoint();
                // Kill
                else
                    playerScoreUIs[killEvent.KilledByPlayer.number].AddPoint();
                yield return new WaitForSecondsRealtime(0.5f);
            }
            
            yield return new WaitForSecondsRealtime(1f);
            
            // Fade out
            while (canvasGroup.alpha > 0f)
            {
                canvasGroup.alpha -= Time.unscaledDeltaTime / 0.25f;
                yield return null;
            }

            if(playerWin)
                ResetScore();
        }
    }
}